unit DateEdit;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Menus, Forms, StdCtrls;

type
  TDateOrder = (MonthDayYear, DayMonthYear, YearMonthDay);
  SelIndex = 0..9;

  TDateEdit = class(TCustomEdit)
  private
    AllDigits: set of SelIndex;
    DayDigit: SelIndex;
    MonthDigit: SelIndex;
    YearDigit: SelIndex;
    Separators: set of SelIndex;
    TOList: TList;
    FSelText: PChar;
    FEditDate: TDateTime;
    FCentury: Boolean;
    FDayLeadingZero: Boolean;
    FEmptyChar: Char;
    FInternational: Boolean;
    FMonthLeadingZero: Boolean;
    FOrder: TDateOrder;
    FSeparator: Char;
    FUseCurrentDate: Boolean;
    procedure SetEditDate(Value: TDateTime);
    procedure SetCentury(Value: Boolean);
    function GetDateElement(Index: integer): integer;
    procedure SetDateElement(Index: integer; Value: integer);
    procedure SetEmptyChar(Value: char);
    procedure SetInternational(Value: boolean);
    procedure SetOrder(Value: TDateOrder);
    procedure SetSeparator(Value: Char);
    procedure SetLeadingZero(Index: integer; Value: boolean);
    procedure SetUseCurrentDate(Value: boolean);
    function IsValidChar(Pos: word; Key: char): boolean;
    function DaysPerMonth(AYear, AMonth: Integer): Integer;
  protected
    procedure CreateParams(var Params: TCreateParams); override;
    procedure CreateWnd; override;
    procedure DoEnter; override;
    procedure DoExit; override;
    procedure KeyPress(var Key: Char); override;
    function ValidateDate: TDateTime; virtual;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure UpdateEdit; virtual;
    property EditDate: TDateTime read FEditDate write SetEditDate;
  published
    property AutoSelect default false;
    property AutoSize;
    property BorderStyle;
    property Century: Boolean read FCentury  write SetCentury;
    property Color;
    property Ctl3D;
    property Day: integer index 1 read GetDateElement write SetDateElement
        stored False;
    property DayLeadingZero: Boolean index 1 read FDayLeadingZero
        write SetLeadingZero;
    property DragCursor;
    property DragMode;
    property EmptyChar: char read FEmptyChar write SetEmptyChar default ' ';
    property Enabled;
    property Font;
    property International: Boolean read FInternational write SetInternational
        default True;
    property Month: integer index 2 read GetDateElement write SetDateElement
        stored False;
    property MonthLeadingZero: Boolean index 2 read FMonthLeadingZero
        write SetLeadingZero;
    property Order: TDateOrder read FOrder write SetOrder;
    property ParentColor;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ReadOnly;
    property Separator: Char read FSeparator write SetSeparator;
    property ShowHint;
    property TabOrder;
    property TabStop;
    property UseCurrentDate: Boolean read FUseCurrentDate
        write SetUseCurrentDate default True;
    property Visible;
    property Year: integer index 3 read GetDateElement write SetDateElement
        stored False;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
  end;

  TTimeEdit = class(TCustomEdit)
  private
    TOList: TList;
    FSelText: PChar;
    FEditTime: TDateTime;
    FEmptyChar: Char;
    FInternational: Boolean;
    FLeadingZero: Boolean;
    FSeparator: Char;
    FUseCurrentTime: Boolean;
    procedure SetEditTime(Value: TDateTime);
    function GetDateElement(Index: integer): integer;
    procedure SetDateElement(Index: integer; Value: integer);
    procedure SetEmptyChar(Value: char);
    procedure SetInternational(Value: boolean);
    procedure SetLeadingZero(Value: boolean);
    procedure SetSeparator(Value: Char);
    procedure SetUseCurrentTime(Value: boolean);
    function IsValidChar(Pos: word; Key: char): boolean;
  protected
    procedure CreateParams(var Params: TCreateParams); override;
    procedure CreateWnd; override;
    procedure DoEnter; override;
    procedure DoExit; override;
    procedure KeyPress(var Key: Char); override;
    function ValidateTime: TDateTime; virtual;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure UpdateEdit; virtual;
    property EditTime: TDateTime read FEditTime write SetEditTime;
  published
    property AutoSelect default false;
    property AutoSize;
    property BorderStyle;
    property Color;
    property Ctl3D;
    property DragCursor;
    property DragMode;
    property EmptyChar: char read FEmptyChar write SetEmptyChar default ' ';
    property Enabled;
    property Font;
    property Hours: integer index 1 read GetDateElement write SetDateElement
        stored False;
    property International: Boolean read FInternational write SetInternational
        default True;
    property LeadingZero: Boolean read FLeadingZero write SetLeadingZero;
    property Minutes: integer index 2 read GetDateElement write SetDateElement
        stored False;
    property ParentColor;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ReadOnly;
    property Separator: Char read FSeparator write SetSeparator;
    property ShowHint;
    property TabOrder;
    property TabStop;
    property UseCurrentTime: Boolean read FUseCurrentTime
        write SetUseCurrentTime default True;
    property Visible;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('Samples', [TDateEdit]);
  RegisterComponents('Samples', [TTimeEdit]);
end;

type
TSelected = record
    Start: word;
    Stop: word;
end;
TSelectedPtr = ^TSelected;

{ --------------------------- TDateEdit-class ------------------------------- }

constructor TDateEdit.Create(AOwner: TComponent);
begin
    inherited Create(AOwner);
    TOList := TList.Create;
    FSelText := StrAlloc(11);
    AutoSelect := false;
    FEmptyChar := ' ';
    FEditDate := Date;
    FUseCurrentDate := true;
    International := true;
end;

destructor TDateEdit.Destroy;
begin
    TOList.Free;
    StrDispose(FSelText);
    inherited Destroy
end;

procedure TDateEdit.CreateParams(var Params: TCreateParams);
begin
    inherited CreateParams(Params);
    Params.Style := Params.Style or ES_MULTILINE or WS_CLIPCHILDREN;
end;

procedure TDateEdit.CreateWnd;
var Loc: TRect;
begin
    inherited CreateWnd;
    SendMessage(Handle, EM_GETRECT, 0, LongInt(@Loc));
    Loc.Bottom := ClientHeight + 1;  {+1 is workaround for windows paint bug}
    Loc.Right := ClientWidth - 1;
    Loc.Top := 0;
    Loc.Left := 0;
    SendMessage(Handle, EM_SETRECTNP, 0, LongInt(@Loc));
end;

procedure TDateEdit.DoEnter;
var sel: TSelectedPtr;
    ret: longint;
begin
    inherited DoEnter;
    sel := TSelectedPtr(@ret);
    sel^.Start := 0;
    if AutoSelect then sel^.Stop := MaxLength
    else sel^.Stop := 0;
    SendMessage(Handle, EM_SETSEL, 0, ret);
end;

procedure TDateEdit.DoExit;
var dt: TDateTime;
begin
    dt := ValidateDate;
    if dt < 0 then begin
        MessageBeep(0);
        UpdateEdit;
        SetFocus
    end else begin
        FEditDate := dt;
        inherited DoExit
    end;
end;

procedure TDateEdit.KeyPress(var Key: Char);
var sel: TSelectedPtr;
    ret: longint;
    dt: TDateTime;
    sl, i: word;
    wc: TWinControl;
    db: TButton;
    bs: boolean;
begin
    if Key = #13 then begin
        wc := self;
        while wc.Parent <> nil do
            wc := wc.Parent;
        wc.GetTabOrderList(TOList);
        db := nil;
        while TOList.Count > 0 do begin
            wc := TWinControl(TOList[0]);
            if (wc is TButton) and ((wc as TButton).Default) then
                db := (wc as TButton);
            TOList.Delete(0)
        end;
        if db <> nil then begin
            dt := ValidateDate;
            if dt >= 0 then begin
                FEditDate := dt;
                db.Click
            end else begin
                MessageBeep(0);
                UpdateEdit;
                SetFocus;
            end
        end
    end
    else begin
        ret := SendMessage(Handle, EM_GETSEL, 0, 0);
        sel := TSelectedPtr(@ret);
        if IsValidChar(sel^.Start, Key) then begin
            sl := sel^.Stop - sel^.Start;
            if Key = char(VK_BACK) then begin
                if sl = 0 then begin
                    Sel^.Start := Sel^.Start - 1;
                    if Sel^.Start in Separators then
                        Sel^.Start := Sel^.Start - 1;
                end;
                bs := true;
                Key := FEmptyChar;
            end;
            if (sel^.Start in Separators) then
                Key := FSeparator;
            if sl = 0 then begin
                sl := 1;
                sel^.Stop := Sel^.Start + sl;
                SendMessage(Handle, EM_SETSEL, 0, ret)
            end;
            if Key = ' ' then
                Key := FEmptyChar;
            FSelText[0] := Key;
            i := 1;
            while i < sl do begin
                if sel^.Start + i in Separators then Key := FSeparator
                else Key := FEmptyChar;
                FSelText[i] := Key;
                Inc(i)
            end;
            FSelText[i] := #0;
            SendMessage(Handle, EM_REPLACESEL, 0, longint(FSelText));
            if not bs then begin
                Inc(sel^.Start);
                if sel^.Start in Separators then Inc(sel^.Start);
            end;
            sel^.Stop := sel^.Start;
            SendMessage(Handle, EM_SETSEL, 0, ret);
        end
        else MessageBeep(0);
    end;
    Key := #0;
    inherited KeyPress(Key);
end;

function TDateEdit.IsValidChar(Pos: word; Key: char): boolean;
begin
    if Key = char(VK_BACK) then Result := true
    else if Pos = DayDigit then Result := (Key in [' ', '0'..'3'])
    else if Pos = MonthDigit then Result := (Key in [' ', '0'..'1'])
    else if Pos in AllDigits then Result := (Key in ['0'..'9'])
    else if Pos in Separators then Result := true
    else Result := false;
end;

procedure TDateEdit.SetEditDate(Value: TDateTime);
begin
    Value := Value - Frac(Value);
    if Value <> FEditDate then begin
        FEditDate := Value;
        UpdateEdit
    end
end;

procedure TDateEdit.UpdateEdit;
var AYear, AMonth, ADay: word;
    AText, tx: string[10];
    i: integer;
    dt, mt: Char;
begin
    AText := '';
    for i := 1 to MaxLength do
        AText := AText + FEmptyChar;
    if FDayLeadingZero then dt := '0'
    else dt := EmptyChar;
    if FMonthLeadingZero then mt := '0'
    else mt := EmptyChar;
    DecodeDate(FEditDate, AYear, AMonth, ADay);
    tx := IntToStr(ADay);
    if ADay >= 10 then begin
        AText[DayDigit + 1] := tx[1];
        AText[DayDigit + 2] := tx[2]
    end else begin
        AText[DayDigit + 1] := dt;
        AText[DayDigit + 2] := tx[1]
    end;
    tx := IntToStr(AMonth);
    if AMonth >= 10 then begin
        AText[MonthDigit + 1] := tx[1];
        AText[MonthDigit + 2] := tx[2]
    end else begin
        AText[MonthDigit + 1] := mt;
        AText[MonthDigit + 2] := tx[1]
    end;
    tx := IntToStr(AYear);
    if Length(tx) <> 4 then Exit;
    if FCentury then
        for i := 1 to 4 do AText[YearDigit + i] := tx[i]
    else
        for i := 1 to 2 do AText[YearDigit + i] := tx[i + 2];
    for i := 1 to MaxLength do
        if (i - 1) in Separators then AText[i] := FSeparator;
    Text := AText;
end;

function TDateEdit.GetDateElement(Index: Integer): Integer;
var
  AYear, AMonth, ADay: Word;
begin
  DecodeDate(FEditDate, AYear, AMonth, ADay);
  case Index of
    1: Result := ADay;
    2: Result := AMonth;
    3: Result := AYear;
    else Result := -1;
  end;
end;

procedure TDateEdit.SetDateElement(Index: Integer; Value: Integer);
var AYear, AMonth, ADay: Word;
    ADate: TDateTime;
begin
    try
        DecodeDate(ADate, AYear, AMonth, ADay);
        case Index of
            1: ADate := EncodeDate(AYear, AMonth, Value);
            2: ADate := EncodeDate(AYear, Value, ADay);
            3: ADate := EncodeDate(Value, AMonth, ADay);
            else Exit;
        end;
        FUseCurrentDate := False;
        SetEditDate(ADate)
    except
        on EConvertError do begin
            MessageBeep(0);
            SetFocus
        end
        else raise
    end;
end;

procedure TDateEdit.SetInternational(Value: boolean);
var i, d, m, y: integer;
    st: string[16];
begin
    if Value <> FInternational then begin
        if Value then begin
            st := LowerCase(GetProfileStr('Intl'#0, 'sShortDate'#0,
                'mm/dd/yyyy'));
            FSeparator := #0;
            for i := 1 to Length(st) do
                if st[i] = 'y' then y := i
                else if st[i] = 'm' then m := i
                else if st[i] = 'd' then d := i
                else if FSeparator = #0 then FSeparator := st[i];
            FCentury := ((y > 2) and (st[y - 3] = 'y'));
            FDayLeadingZero := ((d > 0) and (st[d - 1] = 'd'));
            FMonthLeadingZero := ((m > 0) and (st[m - 1] = 'm'));
            if (m < d) and (m < y) then SetOrder(MonthDayYear)
            else if (d < m) and (m < y) then SetOrder(DayMonthYear)
            else SetOrder(YearMonthDay);
        end;
        FInternational := Value;
    end
end;

procedure TDateEdit.SetCentury(Value: Boolean);
begin
    if Value <> FCentury then begin
        FCentury := Value;
        FInternational := false;
        SetOrder(FOrder);
    end
end;

procedure TDateEdit.SetOrder(Value: TDateOrder);
var st: SelIndex;
begin
    if (Value = MonthDayYear) or (Value = DayMonthYear) then begin
        AllDigits := [1, 4, 6, 7];
        YearDigit := 6;
        if FCentury then AllDigits := AllDigits + [8, 9];
        if Value = MonthDayYear then begin
            MonthDigit := 0;
            DayDigit := 3;
        end else begin
            MonthDigit := 3;
            DayDigit := 0;
        end;
        Separators := [2, 5];
    end else begin
        YearDigit := 0;
        if FCentury then begin
            AllDigits := [0, 1, 2, 3, 6, 9];
            MonthDigit := 5;
            DayDigit := 8;
            Separators := [4, 7];
        end else begin
            AllDigits := [0, 1, 4, 7];
            MonthDigit := 3;
            DayDigit := 6;
            Separators := [3, 5];
        end
    end;
    if FCentury then MaxLength := 10
    else MaxLength := 8;
    FInternational := false;
    FOrder := Value;
    UpdateEdit
end;

procedure TDateEdit.SetSeparator(Value: Char);
begin
    if Value <> FSeparator then begin
        FSeparator := Value;
        FInternational := false;
        UpdateEdit
    end
end;

procedure TDateEdit.SetEmptyChar(Value: char);
begin
    if Value <> FEmptyChar then begin
        FEmptyChar := Value;
        UpdateEdit
    end
end;

procedure TDateEdit.SetLeadingZero(Index: integer; Value: boolean);
begin
    case Index of
    1:  if Value <> FDayLeadingZero then begin
            FDayLeadingZero := Value;
            FInternational := false;
            UpdateEdit
        end;
    2:  if Value <> FMonthLeadingZero then begin
            FMonthLeadingZero := Value;
            FInternational := false;
            UpdateEdit
        end
    end
end;

procedure TDateEdit.SetUseCurrentDate(Value: boolean);
begin
    if Value <> FUseCurrentDate then begin
        FUseCurrentDate := Value;
        if Value then SetEditDate(Date)
    end
end;

function TDateEdit.ValidateDate: TDateTime;
var AYear, AMonth, ADay: word;
begin
    try
        if FCentury then
            AYear := StrToIntDef(Copy(Text, YearDigit + 1, 4), 0)
        else begin
            DecodeDate(Date, AYear, AMonth, ADay);
            AYear := ((AYear div 100) * 100) +
                StrToIntDef(Copy(Text, YearDigit + 1, 2), 0);
        end;
        if Text[MonthDigit + 1] = FEmptyChar then
            AMonth := StrToIntDef(Copy(Text, MonthDigit + 2, 1), 0)
        else
            AMonth := StrToIntDef(Copy(Text, MonthDigit + 1, 2), 0);
        if Text[DayDigit + 1] = FEmptyChar then
            ADay := StrToIntDef(Copy(Text, DayDigit + 2, 1), 0)
        else
            ADay := StrToIntDef(Copy(Text, DayDigit + 1, 2), 0);
        if (AYear > 0) and (AMonth > 0) and (ADay > 0) and (AMonth <= 12)
            and (ADay <= DaysPerMonth(AYear, AMonth)) then
            Result := EncodeDate(AYear, AMonth, ADay)
        else Result := -1;
    except
        on EConvertError do Result := -1;
        else raise
    end;
end;

function TDateEdit.DaysPerMonth(AYear, AMonth: Integer): Integer;
const
    DaysInMonth: array[1..12] of Integer =
        (31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31);
begin
    Result := DaysInMonth[AMonth];
    if (AMonth = 2) and (AYear mod 4 = 0) and ((AYear mod 100 <> 0) or
        (AYear mod 400 = 0)) then Inc(Result);
end;

{ ------------------------- TTimeEdit-class ------------------------------ }

constructor TTimeEdit.Create(AOwner: TComponent);
begin
    inherited Create(AOwner);
    TOList := TList.Create;
    FSelText := StrAlloc(6);
    AutoSelect := false;
    MaxLength := 5;
    FEmptyChar := ' ';
    FEditTime := Time;
    FUseCurrentTime := true;
    International := true;
end;

destructor TTimeEdit.Destroy;
begin
    TOList.Free;
    StrDispose(FSelText);
    inherited Destroy
end;

procedure TTimeEdit.CreateParams(var Params: TCreateParams);
begin
    inherited CreateParams(Params);
    Params.Style := Params.Style or ES_MULTILINE or WS_CLIPCHILDREN;
end;

procedure TTimeEdit.CreateWnd;
var Loc: TRect;
begin
    inherited CreateWnd;
    SendMessage(Handle, EM_GETRECT, 0, LongInt(@Loc));
    Loc.Bottom := ClientHeight + 1;  {+1 is workaround for windows paint bug}
    Loc.Right := ClientWidth - 1;
    Loc.Top := 0;
    Loc.Left := 0;
    SendMessage(Handle, EM_SETRECTNP, 0, LongInt(@Loc));
end;

procedure TTimeEdit.DoEnter;
var sel: TSelectedPtr;
    ret: longint;
begin
    inherited DoEnter;
    sel := TSelectedPtr(@ret);
    sel^.Start := 0;
    if AutoSelect then sel^.Stop := MaxLength
    else sel^.Stop := 0;
    SendMessage(Handle, EM_SETSEL, 0, ret);
end;

procedure TTimeEdit.DoExit;
var dt: TDateTime;
begin
    dt := ValidateTime;
    if dt < 0 then begin
        MessageBeep(0);
        UpdateEdit;
        SetFocus
    end else begin
        FEditTime := dt;
        inherited DoExit
    end;
end;

procedure TTimeEdit.KeyPress(var Key: Char);
var sel: TSelectedPtr;
    ret: longint;
    dt: TDateTime;
    sl, i: word;
    wc: TWinControl;
    db: TButton;
    bs: boolean;
begin
    if Key = #13 then begin
        wc := self;
        while wc.Parent <> nil do
            wc := wc.Parent;
        wc.GetTabOrderList(TOList);
        db := nil;
        while TOList.Count > 0 do begin
            wc := TWinControl(TOList[0]);
            if (wc is TButton) and ((wc as TButton).Default) then
                db := (wc as TButton);
            TOList.Delete(0)
        end;
        if db <> nil then begin
            dt := ValidateTime;
            if dt >= 0 then begin
                FEditTime := dt;
                db.Click
            end else begin
                MessageBeep(0);
                UpdateEdit;
                SetFocus;
            end
        end
    end
    else begin
        ret := SendMessage(Handle, EM_GETSEL, 0, 0);
        sel := TSelectedPtr(@ret);
        if IsValidChar(sel^.Start, Key) then begin
            sl := sel^.Stop - sel^.Start;
            if Key = char(VK_BACK) then begin
                if sl = 0 then begin
                    Sel^.Start := Sel^.Start - 1;
                    if Sel^.Start = 2 then
                        Sel^.Start := Sel^.Start - 1;
                end;
                bs := true;
                Key := FEmptyChar;
            end;
            if sel^.Start = 2 then
                Key := FSeparator;
            if sl = 0 then begin
                sl := 1;
                sel^.Stop := Sel^.Start + sl;
                SendMessage(Handle, EM_SETSEL, 0, ret)
            end;
            if Key = ' ' then
                Key := FEmptyChar;
            FSelText[0] := Key;
            i := 1;
            while i < sl do begin
                if (sel^.Start + i) = 2 then Key := FSeparator
                else Key := FEmptyChar;
                FSelText[i] := Key;
                Inc(i)
            end;
            FSelText[i] := #0;
            SendMessage(Handle, EM_REPLACESEL, 0, longint(FSelText));
            if not bs then begin
                Inc(sel^.Start);
                if sel^.Start = 2 then Inc(sel^.Start);
            end;
            sel^.Stop := sel^.Start;
            SendMessage(Handle, EM_SETSEL, 0, ret);
        end
        else MessageBeep(0);
    end;
    Key := #0;
    inherited KeyPress(Key);
end;


function TTimeEdit.IsValidChar(Pos: word; Key: char): boolean;
begin
    if Key = char(VK_BACK) then Result := true
    else if Pos = 0 then Result := (Key in [' ', '0'..'2'])
    else if Pos in [1, 4] then Result := (Key in ['0'..'9'])
    else if Pos = 3 then Result := (Key in ['0'..'5'])
    else if Pos = 2 then Result := true
    else Result := false;
end;

procedure TTimeEdit.SetEditTime(Value: TDateTime);
begin
    Value := Value - Int(Value);
    if Value <> FEditTime then begin
        FEditTime := Value;
        UpdateEdit
    end
end;

procedure TTimeEdit.UpdateEdit;
var AHours, AMinutes, ASeconds, AMSecs: word;
    AText: TCaption;
begin
    DecodeTime(FEditTime, AHours, AMinutes, ASeconds, AMSecs);
    if AHours >= 10 then
        AText := IntToStr(AHours) + FSeparator
    else begin
        if FLeadingZero then
            AText := '0' + IntToStr(AHours) + FSeparator
        else
            AText := FEmptyChar + IntToStr(AHours) + FSeparator;
    end;
    if AMinutes >= 10 then
        Text := AText + IntToStr(AMinutes)
    else
        Text := AText + '0' + IntToStr(AMinutes);
end;

procedure TTimeEdit.SetInternational(Value: boolean);
begin
    if Value <> FInternational then begin
        if Value then begin
            FSeparator := GetProfileChar('Intl'#0, 'sTime'#0, ':');
            FLeadingZero := (GetProfileInt('Intl'#0, 'iTLZero'#0, 0) > 0);
            UpdateEdit
        end;
        FInternational := Value;
    end
end;

procedure TTimeEdit.SetSeparator(Value: Char);
begin
    if Value <> FSeparator then begin
        FSeparator := Value;
        FInternational := false;
        UpdateEdit
    end
end;

procedure TTimeEdit.SetLeadingZero(Value: boolean);
begin
    if Value <> FLeadingZero then begin
        FLeadingZero := Value;
        FInternational := false;
        UpdateEdit
    end
end;

procedure TTimeEdit.SetEmptyChar(Value: char);
begin
    if Value <> FEmptyChar then begin
        FEmptyChar := Value;
        UpdateEdit
    end
end;

procedure TTimeEdit.SetUseCurrentTime(Value: boolean);
begin
    if Value <> FUseCurrentTime then begin
        FUseCurrentTime := Value;
        if Value then SetEditTime(Time);
    end
end;

function TTimeEdit.GetDateElement(Index: Integer): Integer;
var AHours, AMinutes, ASeconds, AMSecs: word;
begin
  DecodeTime(FEditTime, AHours, AMinutes, ASeconds, AMSecs);
  case Index of
    1: Result := AHours;
    2: Result := AMinutes;
    else Result := -1;
  end;
end;

procedure TTimeEdit.SetDateElement(Index: Integer; Value: Integer);
var AHours, AMinutes, ASeconds, AMSecs: word;
    ADate: TDateTime;
begin
    try
        DecodeTime(ADate, AHours, AMinutes, ASeconds, AMSecs);
        case Index of
            1: ADate := EncodeTime(Value, AMinutes, ASeconds, AMSecs);
            2: ADate := EncodeTime(AHours, Value, ASeconds, AMSecs);
            else Exit;
        end;
        FUseCurrentTime := False;
        SetEditTime(ADate)
    except
        on EConvertError do begin
            MessageBeep(0);
            SetFocus
        end
        else raise
    end;
end;

function TTimeEdit.ValidateTime: TDateTime;
var AHours, AMinutes: word;
begin
    try
        AMinutes := StrToIntDef(Copy(Text, 4, 2), -1);
        if Text[1] = FEmptyChar then
            AHours := StrToIntDef(Copy(Text, 2, 1), -1)
        else
            AHours := StrToIntDef(Copy(Text, 1, 2), -1);
        if (AMinutes >= 0) and (AHours >= 0) and (AHours < 24) and
            (AMinutes < 60) then
            Result := EncodeTime(AHours, AMinutes, 0, 0)
        else Result := -1;
    except
        on EConvertError do Result := -1;
        else raise
    end;
end;

end.
